/*
    init.c
    Copyright (C) 1997-1999 by Richard Dawe

    Start-up and shut-down functions for libwin

    ---

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/* Includes */
#include <stdlib.h>

#include <libwin/init.h>
#include <libwin/user.h>

/* Have we init'd the library yet? */
static int win_inited = 0;

/* ------------
   - win_init -
   ------------ */

/* This sets up anything that needs to be set up before the library is used,
   and calls atexit() so that the library cleans up after itself. */

int win_init (void)
{ 
    if (!win_inited) {
        /* Install atexit handler */
        if (atexit(win_uninit) != 0) return(-1);

        /* Set USER and LOGIN variables if they're not already set - not that
           this only does it for this program's environment */
        setenv("USER", w95_getlogon(), 1);
        setenv("LOGIN", w95_getlogon(), 1);
        setenv("LOGNAME", w95_getlogon(), 1);
        setenv("HOME", w95_getprofilepath(w95_getlogon()), 1);

        /* Done now */
        win_inited = 1;
    }

    /* OK */
    return(win_inited);
}

/* --------------
   - win_uninit -
   -------------- */

void win_uninit (void) { }
