/*
    lfn.c
    Copyright (C) 1997-1999 by Richard Dawe

    Long filename functions - part of libwin.

    ---

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/* Inclusions */
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <dpmi.h>
#include <go32.h>
#include <sys/movedata.h>

#include <libwin/lfn.h>

/* Defines */
#define LFN_BUFFER_LEN      262     /* LFN buffer length; max filename = 262 */

/* ----------------
   - w95_lfntosfn -
   ---------------- */

int w95_lfntosfn (const char *lfn, char *sfn)
{
    __dpmi_regs r;          /* Registers for function call  */
    int segment, selector, paras;

    /* Check the LFN isn't too long */
    if ((strlen(lfn) + 1) > LFN_BUFFER_LEN) return(-1);

    /* Allocate some memory - twice size of LFN for returned one */
    paras = (LFN_BUFFER_LEN + 15) >> 4;
    segment = __dpmi_allocate_dos_memory(2 * paras, &selector);

    /* Copy strings */
    movedata(_my_ds(), (int) lfn, selector, 0, strlen(lfn) + 1);

    /* Set up the registers */
    memset (&r, 0, sizeof(r));

    r.x.ax = 0x7160;    /* Get short file name          */
    r.x.cx = 0x0001;    /* Return full path - no SUBSTs */

    r.x.ds = segment;
    r.x.si = 0;
    r.x.es = segment;
    r.x.di = LFN_BUFFER_LEN;

    /* Interrupt */
    __dpmi_int (0x21, &r);

    /* Check for an error */
    if ((r.x.flags & 1) == 1) return(r.x.ax);

    /* Copy SFN from buffer to buffer */
    movedata(selector, LFN_BUFFER_LEN, _my_ds(), (int) sfn, LFN_BUFFER_LEN);

    /* Free memory */
    __dpmi_free_dos_memory(selector);

    /* Return result */
    return(0);
}

/* -------------
   - issfnchar -
   ------------- */

/* This checks to see if the character specified is a valid SFN character. */

int issfnchar (int c)
{
    if (
           isalnum(c)
        || (c == '$') || (c == '%') || (c == '\'') || (c == '-')
        || (c == '_') || (c == '@') || (c == '~')  || (c == '`')
        || (c == '!') || (c == '(') || (c == ')')  || (c == '^')
        || (c == '#') || (c == '&')
       )
        return(1);
    else
        return(0);
}

/* -------------
   - islfnchar -
   ------------- */

int islfnchar (int c)
{
    if (
           issfnchar(c)
        || (c == '+') || (c == ',') || (c == ';') || (c == '=')
        || (c == '=') || (c == '[') || (c == ']')
       )
        return(1);
    else
        return(0);
}
