/*
    user.c
    Copyright (C) 1998, 1999 by Richard Dawe

    User information routines - part of libwin.

    ---

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/* Inclusions */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <libwin/registry.h>
#include <libwin/user.h>

#define LFN_BUFFER_LEN 262

/* ----------------
   - w95_getlogon -
   ---------------- */

/* This retrieves the currently logged-on user's name from the registry key
   "HKEY_LOCAL_MACHINE\Network\Logon\username". */

char *w95_getlogon (void)
{
    long hkey;                          /* Registry key handle      */
    char buffer[256];                   /* Buffer for data          */
    long buffersize = sizeof(buffer);   /* Size of da buffer        */
    long buffertype = REG_SZ;           /* I'd like a string please */

    /* Clear the buffer just in case */
    memset(buffer, 0, buffersize);

    /* Obtain the registry key handle */
    if (w95_reg_openkey(HKEY_LOCAL_MACHINE, "Network\\Logon", &hkey) != 0)
        return(NULL);

    /* Get the current user's logon name */
    if (w95_reg_queryvalueex(hkey, "username", NULL, &buffertype, buffer, &buffersize) != 0)
        return(NULL);

    /* Use strdup to copy the name, and return the pointer */
    return(strdup(buffer));
}

/* ----------------------
   - w95_getprofilepath -
   ---------------------- */

/* This returns the path to the specified user's profile, e.g.
   C:\Windows\Profiles\<username> */

char *w95_getprofilepath (char *user)
{
    char *buffer, *p;

    /* Allocate the memory */
    buffer = malloc(sizeof(char) * LFN_BUFFER_LEN);
    if (buffer == NULL) return(NULL);

    /* Build the path and return it */
    sprintf(buffer, "%s/Profiles/%s", getenv("windir"), user);
    for (p = buffer; *p != '\0'; p++) { if (*p == '\\') *p = '/'; }
    return(buffer);
}
