/*
    vxd.c
    Copyright (C) 1997-2000 by Richard Dawe

    VxD calling functions - part of libwin.

    ---

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/*
 * win_vxd_getentry, win_vxd_call are from code from Dan Hedlund's wsock
 * library, which I took from Indrek Mandre's libsocket library. Modifications
 * to these two functions are indicated by comments beginning with "RD:". Also
 * see the change list below.
 */

/* RD: Summary of changes:

    . Removed unnecessary inclusion
    . Changed function name from VxdGetEntry to win_vxd_getentry
    . Changed comments into my style
    . Added some comments
    . Added an error return if the call fails
    . Added VxD calling function from "callvxd.c"
    . Added a check to win_vxd_call() that the entry point isn't 0:0 -> GPF
    . Added ability to use all registers in win_vxd_call()
    . Added win_vxd_getentrybyname
    . Added win_vxd_getentrybyid as alternative name for win_vxd_getentry
    . Changed win_vxd_getentrybyname to work in same way to other win_vxd_*()
*/

/* Inclusions */
/*#include <dos.h>*/    /* Included in vxd.h */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <dpmi.h>
#include <sys/movedata.h>
#include <sys/segments.h>

#include <libwin/vxd.h>

static int GlobalEntry[2];
union REGS rc;

static short rc_es;
static long rc_eax, rc_ebx, rc_ecx, rc_edx, rc_edi, rc_esi;

/* --------------------
   - win_vxd_getentry -
   -------------------- */                      /* RD: Added comment */

int win_vxd_getentry (int *Entry, int ID)       /* RD: Changed name  */
{
  asm ("pushl   %%es            \n\
        movw    %%di, %%es      \n\
        int     $0x2f           \n\
        movl    $0, %%ecx       \n\
        movw    %%es, %%cx      \n\
        popl    %%es"
        : "=c" (Entry[1]), "=D" (Entry[0])
        : "a" (0x1684), "b" (ID), "D" (0)
      );

    /* RD: Added error checking - returns -1 on failure */
    if ( (Entry[0] == 0) && (Entry[1] == 0) ) return(-1); else return(0);
}

/* ------------------------
   - win_vxd_getentrybyid -
   ------------------------ */

int win_vxd_getentrybyid (int *Entry, int ID) { return(win_vxd_getentry(Entry, ID)); }

/* ----------------
   - win_vxd_call -
   ---------------- */

int win_vxd_call (int *Entry, union REGS *r, struct SREGS *sr)
{
    /* RD: Check that we have an entry point - if not, fail returning -1.
       If this is not done, then the call may cause a protection fault. */
    if ( (Entry[0] == 0) && (Entry[1] == 0) ) return(-1);

    /* Check that the regs have been specified. Segment registers are
       optional. */
    if ( (r == NULL) || (sr == NULL) ) return(-1);

    /* RD: It appears that a global variable must be used for the call -
       how strange. Otherwise -> protection fault! LATER: I found this in
       some doc's somewhere - maybe a tutorial or the gcc / gas doc's. */
    GlobalEntry[0] = Entry[0]; GlobalEntry[1] = Entry[1];

    /* Copy r, sr into rc_* as the inline assembler doesn't like pointer
       dereferences */
    rc_es  = sr->es;
    rc_eax = r->d.eax;
    rc_ebx = r->d.ebx;
    rc_ecx = r->d.ecx;
    rc_edx = r->d.edx;
    rc_edi = r->d.edi;
    rc_esi = r->d.esi;

    /* Perform the call - this isn't particularly elegant code, but it does
       the job (hopefully). */
    asm volatile (" pushl   %%es                                \n\
                                                                \n\
                    # Fill the regs                             \n\
                    movw    _rc_es,  %%es                       \n\
                    movl    _rc_eax, %%eax                      \n\
                    movl    _rc_ebx, %%ebx                      \n\
                    movl    _rc_ecx, %%ecx                      \n\
                    movl    _rc_edx, %%edx                      \n\
                    movl    _rc_edi, %%edi                      \n\
                    movl    _rc_esi, %%esi                      \n\
                                                                \n\
                    # Make the call                             \n\
                  "
#if    (GAS_MAJOR == 2) \
    && ((GAS_MINOR < 9) || ((GAS_MINOR == 9) && (GAS_MINORMINOR < 5)))
                  " lcall   _GlobalEntry                        \n\
                  "
#else
                  " lcall  *_GlobalEntry                        \n\
                  "
#endif
                  "                                             \n\
                    # Copy it all back                          \n\
                    movw    %%es,  _rc_es                       \n\
                    movl    %%eax, _rc_eax                      \n\
                    movl    %%ebx, _rc_ebx                      \n\
                    movl    %%ecx, _rc_ecx                      \n\
                    movl    %%edx, _rc_edx                      \n\
                    movl    %%edi, _rc_edi                      \n\
                    movl    %%esi, _rc_esi                      \n\
                                                                \n\
                    # Restore %%es                              \n\
                    popl    %%es"
                    : :
                    /* The reason %%es isn't in the clobbered list is that the
                       compiler didn't seem to like it */
                    : "%eax", "%ebx", "%ecx", "%edx", "%edi", "%esi",
                      "memory", "cc" );

    /* Put the results back in r */
    sr->es   = rc_es;
    r->d.eax = rc_eax;
    r->d.ebx = rc_ebx;
    r->d.ecx = rc_ecx;
    r->d.edx = rc_edx;
    r->d.edi = rc_edi;
    r->d.esi = rc_esi;

    /* Return contents of AX register */
    return(r->x.ax);
}

/* RD: The following comment refers to some code that Alfons Hoogervorst sent
   me. Where it talks about "program", it is referring to the code that was
   originally the basis for my function win_vxd_getentrybyname(). That function
   doesn't particularly resemble his function anymore, but he contributed how
   to do it in the first place. */

/* vxdproc - Gets the VxD proc using an undocumented feature of Windows (??)
 * By     : Alfons Hoogervorst / Alliterated Software Development, 1998.
 * E-mail : proteus@hoogervorst.demon.nl
 *
 * If you find this information in this program useful, and you feel about
 * donating some money to me, *DON'T* do that. Instead provide input and
 * feedback; supply and contribute code, preferably in source code form.
 * (However, if you insist on donating money, OK, go ahead. :-)
 *
 * This program first appeared in the mailing list for WSock related
 * programming, set up and maintained by Jacob Verhoeks /
 * <jacob@duoware.demon.nl>. Send him a mail if you want to subscribe.
 * The DosSock mailing list is not heavily trafficked, but you'll find
 * there all knowledgeable on the WSOCK.VXD and WinSock.
 */

/* DESCRIPTION: Rich N. Dawe <rd5718@bristol.ac.uk> noted problems with
 * clashing VxD IDs. This file shows an alternative way to get a VxD proc
 * if you know the VxD's name. It uses the same "Get VxD Entry Point",
 * but instead of supplying a VxD ID in BX, you clear BX and let ES:DI
 * point to null-terminated string of max. 8 chars wide (right-padded
 * with blanks).
 */

/* RD: . vxdproc -> win_vxd_getentrybyname
       . Added storage of entry-point into Entry
       . int returned instead of void *
       . Modified assembly into AT&T format
       . Takes normal VxD name and then processes it.
*/

int win_vxd_getentrybyname (int *Entry, char *name)
{
    char *stem, *p;                 /* VxD name stem, temp. pointer */
    char nulls[9];                  /* 8 blanks + NULL              */

    /* Chop the extension off name, if necessary */
    stem = strdup(name);
    if ( (p = strchr(stem, '.')) != NULL) *p = '\0';

    /* Construct a null-padded string */
    memset(nulls, ' ', sizeof(nulls)); nulls[sizeof(nulls) - 1] = 0;
    memcpy(nulls, stem, strlen(stem));

    /* Perform the call */
    asm volatile ("     pushw %%es              \n\
                        pushl %%edi             \n\
                        movw  %%bx, %%es        \n\
                        movl  %%ecx, %%edi      \n\
                        xorl  %%ebx, %%ebx      \n\
                        xorl  %%ecx, %%ecx      \n\
                        xorl  %%edx, %%edx      \n\
                        int   $0x2F             \n\
                        movw  %%es, %%bx        \n\
                        movw  %%di, %%cx        \n\
                        popl  %%edi             \n\
                        popw  %%es"
                    : "=b" (Entry[1]), "=c" (Entry[0])
                    : "a" (0x1684), "b" (_my_ds()),  "c" ((int) nulls)
                    : "%edx"
                 );

    /* Free memory */
    free(stem);

    /* Check the entry-point's OK */
    if ( (Entry[0] == 0) && (Entry[1] == 0) ) return(-1); else return(0);
}
