/*
    vxdldr.c
    Copyright (C) 1997-1999 by Richard Dawe

    VxD loading functions - part of libwin

    ---

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/*
    This code was taken from libsocket 0.6 by Indrek Mandre. It has been
    modified - modifications are indicated by start & end comments with
    "RD:" at the front.
*/

/*
 * Original code from Dan Hedlunds wsock library.
 */

/* RD: Summary of changes:

    . Changed references to VxdGetEntry to w95_vxd_getentry
    . Changed function names to same style as rest of library
    . Removed inclusion of "vxd.h" and "vxdldr.h" (superfluous)
    . Removed inclusion of "farptrx.h" and put its contents in this file
    . Removed inclusion of "glob.h" and put some of it in this file
    . Added some comments to change into my style
    . Capitalised constants
    . Modified win_vxdldr_init to return win_vxd_load's return codes
    . Add some error checking to make return values consistent with other funcs
*/

/* Inclusions */
#include <dpmi.h>
#include <string.h>
#include <stdio.h>

#include <sys/movedata.h>
#include <sys/segments.h>

#include <libwin/vxd.h>

/* RD: Defines from "farptrx.h" */
#define _farpokex(sel,off,dat,len) movedata(_my_ds(),(int)dat,sel,off,len)
#define _farpeekx(sel,off,dat,len) movedata(sel,off,_my_ds(),(int)dat,len)
/* RD: End of stuff from "farptrx.h" */

/* Defines */
#define VXDLDR_FUNC_GETVERSION      0
#define VXDLDR_FUNC_LOADDEVICE      1
#define VXDLDR_FUNC_UNLOADDEVICE    2   /* This appears to be the last
                                           supported service under '95 - all
                                           others returned 0x0B = unsupp'd   */

/* Function declarations */
int win_vxdldr_init (void);

/* Globals */
int vxdldr_entry[2] = {0, 0};

/* -------------------
   - win_vxdldr_init -
   ------------------- */

int win_vxdldr_init (void)
{
    return(win_vxd_getentry(vxdldr_entry, VXDID_VXDLDR));
}

/* ----------------
   - win_vxd_load -
   ---------------- */

int win_vxd_load (char Device[])
{
  int Error;

  __dpmi_meminfo _dev;
  int dev;

  /* RD: Check that we have obtained the entry point, and fail if we can't */
  if ( (vxdldr_entry[0] == 0) && (vxdldr_entry[1] == 0) ) win_vxdldr_init();
  if ( (vxdldr_entry[0] == 0) && (vxdldr_entry[1] == 0) ) return(-1);

  _dev.handle = 0;
  _dev.size = strlen (Device) + 1;
  _dev.address = 0;

  __dpmi_allocate_memory (&_dev);
  dev = __dpmi_allocate_ldt_descriptors (1);
  __dpmi_set_segment_base_address (dev, _dev.address);
  __dpmi_set_segment_limit (dev, _dev.size);

  _farpokex (dev, 0, Device, _dev.size);

  asm ("pushl   %%ds            \n\
        movw    %%cx, %%ds      \n\
       "
#if    (GAS_MAJOR == 2) \
    && ((GAS_MINOR < 9) || ((GAS_MINOR == 9) && (GAS_MINORMINOR < 5)))       
       "lcall   %%cs:_vxdldr_entry \n\
       "
#else
       "lcall  *%%cs:_vxdldr_entry \n\
       "       
#endif
       "setc    %%al            \n\
        movzbl  %%al, %%eax     \n\
        popl    %%ds"
        : "=a" (Error)
        : "a" (VXDLDR_FUNC_LOADDEVICE), "c" (dev), "d" (0));

  __dpmi_free_ldt_descriptor (dev);
  __dpmi_free_memory (_dev.handle);

    return(Error);
}

/* ------------------
   - win_vxd_unload -
   ------------------ */

int win_vxd_unload (char Device[])
{
  int Error;

  __dpmi_meminfo _dev;
  int dev;

  if (vxdldr_entry[1] == 0) win_vxdldr_init();
  if (vxdldr_entry[1] == 0) return(-1);

  _dev.handle = 0;
  _dev.size = strlen (Device) + 1;
  _dev.address = 0;

  __dpmi_allocate_memory (&_dev);
  dev = __dpmi_allocate_ldt_descriptors (1);
  __dpmi_set_segment_base_address (dev, _dev.address);
  __dpmi_set_segment_limit (dev, _dev.size);

  asm ("pushl   %%ds            \n\
        movw    %%cx, %%ds      \n\
       "
#if    (GAS_MAJOR == 2) \
    && ((GAS_MINOR < 9) || ((GAS_MINOR == 9) && (GAS_MINORMINOR < 5)))       
       "lcall   %%cs:_vxdldr_entry \n\
       "
#else
       "lcall  *%%cs:_vxdldr_entry \n\
       "       
#endif
       "setc    %%al            \n\
        movzbl  %%al, %%eax     \n\
        popl    %%ds"
        : "=a" (Error)
        : "a" (VXDLDR_FUNC_UNLOADDEVICE), "c" (dev), "d" (0));

  __dpmi_free_ldt_descriptor (dev);
  __dpmi_free_memory (_dev.handle);

  return(Error);
}
